#!/bin/bash
#
#Description: build and package the project script.
#Date       : 2012-01-06
#Author     : zhangjw@ucweb.com
#CopyRight  : www.ucweb.com

#-------------------------------------------------------------
#-------------------------------------------------------------
#                Utility functions here
#-------------------------------------------------------------
#-------------------------------------------------------------

#-------------------------------------------------------------
#   int varibales of directory.
#-------------------------------------------------------------
init_path_vars()
{ 
BaseDir=`pwd`
ScriptDir="$BaseDir/build_scripts_ipad"

projectfile=U3iPad_Shell_int.xcodeproj
infolistfile=UCMobileIpad-Info_int.plist

#build dirs.
Release="Release-iphoneos"
ReleaseSim="Release-iphonesimulator"

Debug="Debug-iphoneos"
DebugSim="Debug-iphonesimulator"

Distribution="Distribution-iphoneos"
DistributionSim="Distribution-iphonesimulator"

#the root dir of xcode project.
DIR_SRC_ROOT="$BaseDir"

DIR_IOS="$DIR_SRC_ROOT/BrowserShell_ios"
#the the dir of ipad project file.
DIR_PROJECT="$DIR_SRC_ROOT/BrowserShell_ios/ipad-cn/projects"
DIR_CORE_PROJECT="$DIR_SRC_ROOT/BrowserShell_ios/CoreProjects"

DIR_BUILD="$DIR_PROJECT/build"
DIR_UCWEB_BUILD="$DIR_IOS/CoreProjects/build"

DIR_BUILD_RELEASE="$DIR_BUILD/$Release"
DIR_BUILD_RELEASE_SIM="$DIR_BUILD/$ReleaseSim"

DIR_BUILD_DEBUG="$DIR_BUILD/$Debug"
DIR_BUILD_DEBUG_SIM="$DIR_BUILD/$DebugSim"

DIR_BUILD_DISTRIBUTION="$DIR_BUILD/$Distribution"
DIR_BUILD_DISTRIBUTION_SIM="$DIR_BUILD/$DistributionSim"

DIR_UCWEB_BUILD_RELEASE="$DIR_UCWEB_BUILD/$Release"
DIR_UCWEB_BUILD_RELEASE_SIM="$DIR_UCWEB_BUILD/$ReleaseSim"

DIR_UCWEB_BUILD_DEBUG="$DIR_UCWEB_BUILD/$Debug"
DIR_UCWEB_BUILD_DEBUG_SIM="$DIR_UCWEB_BUILD/$DebugSim"

DIR_UCWEB_BUILD_DISTRIBUTION="$DIR_UCWEB_BUILD/$Distribution"
DIR_UCWEB_BUILD_DISTRIBUTION_SIM="$DIR_UCWEB_BUILD/$DistributionSim"

#the dir for SoftInfo.h
DIR_SOFTINFO="$DIR_PROJECT/.."
#the dir for build result
DIR_RESULT="$DIR_SRC_ROOT/build_result"
DIR_OUT="$DIR_SRC_ROOT/OUT"

}

#-------------------------------------------------------------
#   int varibales of file.
#-------------------------------------------------------------
init_file_vars()
{
    SoftInfo="SoftInfo.h"
    FeaturesFile="Features.xcconfig"
    FeaturesPath=$DIR_CORE_PROJECT/$FeaturesFile

    ProductName="UCBrowserHD"
    appName=$ProductName".app"

    BuildScript="BuildUCM.sh"
    PkgScritp="ucweb.sh"

}

#-------------------------------------------------------------
#   int varibales of file.
#-------------------------------------------------------------
init_other_vars()
{
    MACRO_ENABLE_APPSTORE="ENABLE_APPSTORE"
    MACRO_ENABLE_MOKEY_TEST="ENABLE_MONKEY_TEST"
    MACRO_ENABLE_IOSURFACE="WTF_USE_IOSURFACE_CANVAS_BACKING_STORE"
    MACRO_ENABLE_PERFORMANCE_TEST="UC_ENABLE_PERFORMANCE_TEST"
    ENABLE_LOCAL_CRASHLOG="ENABLE_LOCAL_CRASHLOG"
    MACRO_ENABLE_CRASH_LOG="ENABLE_CRASH_LOG"
    MACRO_ENABLE_CLOSE_BETA_TEST="ENABLE_CLOSE_BETA_TEST"
}


#-------------------------------------------------------------
#   generate the log.
#-------------------------------------------------------------
function uclog()
{   
    if [ $# -ne "0" ]
    then 
        MSG=$*
       OUTMSG="--- $MSG"
       echo $OUTMSG >> ~/build_err/err.log
    fi
}


#-------------------------------------------------------------
#  transform "/" into "\/"
#  return value is $ret_url
#-------------------------------------------------------------
escape_url()
{
    len=`echo "$1"|wc -c`
    len=$((len-1))
    i=0
    ret_url=""
    while [ ! "$i" -ge $len ]
    do
     cur_char=${1:$i:1}
     if [ $cur_char = "/" ]
     then
        #echo "***\***"
        ret_url=$ret_url\\
     fi
     #echo $cur_char
     ret_url=$ret_url$cur_char
     i=$((i+1))
    done
}


#-------------------------------------------------------------
# e.g.: #define UC_SERVER_URL1  "http://u.uc123.com:80/"
#-------------------------------------------------------------
modify_url()
{
    key=$1

    escape_url $2

    NEWURL="$key    \""
    NEWURL=$NEWURL$ret_url\"
    echo $NEWURL

    sed -i bak  "1,/$key/{s/$key.*/$NEWURL/;}" $VER_DEFINE_FILE
}

#-------------------------------------------------------------
# modify the macro value for key.
# e.g.: #define UC_SET_VER  "8.0.0.89"
#-------------------------------------------------------------
modify_value()
{
    key=$1
    value=$2

    NEWVAL="$key    \""
    NEWVAL=$NEWVAL$value\"
    uclog "NEWVAL=$NEWVAL"

    sed  -i bak "1,/$key/{s/$key*[ ].*/$NEWVAL/;}" $VER_DEFINE_FILE
}


#-------------------------------------------------------------
#   set the the variables for SoftInfo.h
#-------------------------------------------------------------
function to_soft_vars()
{
    DEFS_VAR=$1
    uclog "DEFS_VAR:$DEFS_VAR"
    
    SOFT_VAR=""
    
    if   [ "$DEFS_VAR" = "DEFS_UC_SET_VER" ]; then 
            SOFT_VAR="SOFT_VER"
    elif [ "$DEFS_VAR" = "DEFS_UC_BID" ]; then
            SOFT_VAR="SOFT_BRAND_ID"
    elif [ "$DEFS_VAR" = "DEFS_UC_BUILDDATA" ]; then
            SOFT_VAR="SOFT_BUILD_SEQ"
    elif [ "$DEFS_VAR" = "DEF_UC_LANG" ]; then
            SOFT_VAR="SOFT_LANG"
    elif [ "$DEFS_VAR" = "DEFS_BTYPE" ]; then
            SOFT_VAR="SOFT_BTYPE"
    elif [ "$DEFS_VAR" = "DEFS_BMODE" ]; then
            SOFT_VAR="SOFT_BMODE"
    else
            SOFT_VAR=$DEFS_VAR
    fi
    
    uclog "SOFT_VAR:$SOFT_VAR"
}


#-------------------------------------------------------------
# add new macro key-value pair
# e.g.: #define    UC_USING_M9
#-------------------------------------------------------------
append_value()
{
    oldDefCount=`grep -o "^#define.*$1" $VER_DEFINE_FILE|wc -l`

    if [  $oldDefCount -eq 0 ]; then
        line="#define  "$1
        echo $line

        echo $line >> $VER_DEFINE_FILE
    fi
}

#-------------------------------------------------------------
# cancel the line inclues $key with "//"
# e.g.: //#define    UC_USING_M9
#-------------------------------------------------------------
comment_value()
{
    oldDefCount=`grep -o "^#define.*$1" $VER_DEFINE_FILE|wc -l`

    if [  $oldDefCount -eq 1 ]; then
        sed -i bak  "/^#define.*$1/{s/^/\/\//;}" $VER_DEFINE_FILE
    fi
}

#-------------------------------------------------------------
#  return value of plistkey in $pvalue
#-------------------------------------------------------------
get_plist_value()
{
    key=$1
    file=$2

    pvalue=`grep $key -A 1 $file|grep -G -o '<string>.*</string>'|sed 's/<.\{1,2\}tring>//g'`
}


#-------------------------------------------------------------
#  modify value of plistkey 
#-------------------------------------------------------------
modify_plist_value()
{
    key=$1
    value=$2
    file=$3

    oldvalue=`grep $key -A 1 $file|grep -G -o '<string>.*</string>'|sed 's/<.\{1,2\}tring>//g'`
    sed -i bak "s/$oldvalue/$value/" $file
}

#-------------------------------------------------------------
#   modify the ***info.plist
#-------------------------------------------------------------
modify_infolist()
{   
    INFOFILE=$1

    if [ -f $INFOFILE ]; then

         ####---- CFBundleIdentifier
         get_plist_value "CFBundleIdentifier" $INFOFILE

         ####---- CFBundleVersion
         modify_plist_value "CFBundleVersion" $DEFS_UC_SET_VER $INFOFILE
    else
         uclog "can't found INFOFILE"
         exit 88
    fi
}

#------------------------------------------------------------
#   modify bundle
#-------------------------------------------------------------
modify_bundle()
{
    INFOFILE=$1
    if [ -f $INFOFILE ]; then
        if [ "$DEF_CERT_EE" = "true" ]; then
            modify_plist_value 'com.starlight.hd' 'com.ucbrowser.ipad.en' $INFOFILE
        elif [ $DEF_TYPE3 = "true" ]; then
            modify_plist_value 'com.starlight.hd' 'com.ucweb.ipad.int' $INFOFILE
        fi
    else
         uclog "can't found INFOFILE"
         exit 88
    fi
}

##############################################################
#  copy *.a ---> OUT dir
##############################################################
copy_lib_file()
{
    LIB_FILE=$1

    if [ $DEF_TYPE1 = "true" ]; then
         srcLibFile="build/Products/Release-iphoneos/"$LIB_FILE
    elif [ $DEF_TYPE2 = "true" ]; then
         srcLibFile="build/Products/Debug-iphoneos/"$LIB_FILE
    else
         srcLibFile="build/Products/Release-iphoneos/"$LIB_FILE
    fi

    cp $srcLibFile ./OUT
}

##############################################################
#  copy *.a ---> OUT dir (Distribution Version)
##############################################################
copy_dist_lib_file()
{
    LIB_FILE=$1

    if [ $DEF_TYPE1 = "true" ]; then
         srcLibFile="build/Products/Release-iphonesimulator/"$LIB_FILE
    elif [ $DEF_TYPE2 = "true" ]; then
         srcLibFile="build/Products/Debug-iphonesimulator/"$LIB_FILE
    elif [ $DEF_TYPE3 = "true" ]; then
        if [ $DEF_RUN1 = "true" ]; then
            srcLibFile="build/Products/Release-iphoneos/"$LIB_FILE
        else
            srcLibFile="build/Products/Release-iphonesimulator/"$LIB_FILE
        fi
    else
        srcLibFile="build/Products/Release-iphonesimulator/"$LIB_FILE
    fi
    
    cp $srcLibFile ./OUT
}

enable_layout_test()
{
  uclog "enable_layout_test"
    if [ $DEF_LAYOUT_TEST = "true" ]; then
      uclog "enable_layout_test+++++"
        (cd "$BaseDir/Tools/DumpRenderTree/res/ios/"

        if [ ! -x ./drt_copy_files ]; then
        chmod +x ./drt_copy_files
        fi
        ./drt_copy_files
        )

        MacroName=$"ENABLE_LAYOUTTEST"
        sed -i bak "s/\/\/$MacroName/$MacroName/" $FeaturesPath
    fi
}

enable_remote_inspector()
{
    uclog "enable_remote_inspector"
    if [ $DEF_REMOTE_INSPECTOR = "true" ]; then
    uclog "enable_remote_inspector+++++"
    (cd "$DIR_IOS"
        sh copy_inspector_frontend
    )

    MacroName=$"ENABLE_INSPECTOR"
    sed -i bak "s/\/\/$MacroName/$MacroName/" $FeaturesPath
fi
}

#-------------------------------------------------------------
#  modify the value in config.ini for u-alliance.
#-------------------------------------------------------------
modify_set_value()
{
    key=$1
    value=$2
    configFilePath=$3

    NEWVAL="$key="$value
    echo $NEWVAL
    sed  -i bak "1,/$key/{s/$key=.*/$NEWVAL/;}" $configFilePath
}

#-------------------------------------------------------------
#-------------------------------------------------------------
#                main entrance functions
#-------------------------------------------------------------
#-------------------------------------------------------------

#-------------------------------------------------------------
#   init the build env variables
#-------------------------------------------------------------
init_vars()
{
    uclog ">>> $BuildScript : init_vars()"
    init_path_vars
    init_file_vars
    init_other_vars
}

printVars()
{
  uclog $project
  uclog $DEFS_UC_SERVER_URL1

}

#-------------------------------------------------------------
#  init the build env variables
#-------------------------------------------------------------
init_env()
{
    uclog ">>> $BuildScript : init_env()"

    cat build.ini |grep -v "#" |grep -v "(" |sed 's/^M$//' |sed 's/\/$//' |sed '/=/s/^/export /' > buildenv.sh
    . ./buildenv.sh

    export VER_DEFINE_FILE=$SoftInfo
    export CURDIR=`pwd`
}

#-------------------------------------------------------------
#  alter SoftInfo.h 
#-------------------------------------------------------------
alter_softInfo()
{
    uclog ">>> $BuildScript : alter_softInfo()"

    if [ ! -f $DIR_SOFTINFO/$SoftInfo ]
    then
        echo "$DIR_SOFTINFO/$SoftInfo not found!"
        exit 99
    fi
    
    cp $DIR_SOFTINFO/$SoftInfo ./
    
#   version.
    if [ ! -z $DEFS_UC_SET_VER ]
    then
        to_soft_vars "DEFS_UC_SET_VER"
        modify_value $SOFT_VAR $DEFS_UC_SET_VER
    fi
    
#   bid.
    if [ ! -z $DEFS_UC_BID ] 
    then 
        to_soft_vars "DEFS_UC_BID"
        modify_value $SOFT_VAR $DEFS_UC_BID
    fi

#   language.
    if [ ! -z $DEF_UC_LANG ]
    then
        to_soft_vars "DEF_UC_LANG"
        modify_value $SOFT_VAR $DEF_UC_LANG
    fi 

#   u-alliance
    if [ "$DEF_U_ALLIANCE" = "true" ]
    then 
        if [ ! -z $DEFS_BMODE ]
        then
            to_soft_vars "DEFS_BMODE"
            modify_value $SOFT_VAR $DEFS_BMODE
        fi
       
        if [ ! -z $DEFS_BTYPE ]
        then 
            to_soft_vars "DEFS_BTYPE"
            modify_value $SOFT_VAR $DEFS_BTYPE
        fi
    fi

#   build_seq.
    export DEFS_UC_BUILDDATA=`date +%y%m%d%H`
    if [ ! -z $DEFS_UC_BUILDDATA ]
    then
        to_soft_vars "DEFS_UC_BUILDDATA"
        modify_value $SOFT_VAR $DEFS_UC_BUILDDATA
    fi

#   the following config is optional.

#   m9.
    append_value "UC_USING_M9"

    if [ ! -d $DIR_RESULT ]
    then
        mkdir $DIR_RESULT
    fi

#   move SoftInfo.h into xcode project directory.
    mv -f $VER_DEFINE_FILE  $DIR_SOFTINFO/$SoftInfo 
    
}

#-------------------------------------------------------------
#   alter the project infolist: UCMobileIpad-Info.plist
#-------------------------------------------------------------
alter_proInfo()
{
    uclog ">>> $BuildScript : alter_proInfo()"
    modify_infolist "$DIR_PROJECT/../$infolistfile"
}

#-------------------------------------------------------------
#   remove MobileInstallation.framework from project.
#-------------------------------------------------------------
function remove_framework()
{
    uclog "remove_framework()"
    cat $DIR_PROJECT/$projectfile/project.pbxproj | grep -v MobileInstallation > $DIR_PROJECT/$projectfile/appstore.pbxproj
    cp -f $DIR_PROJECT/$projectfile/appstore.pbxproj $DIR_PROJECT/$projectfile/project.pbxproj
}

#-------------------------------------------------------------
#   enable the macro ENABLE_APPSTORE in featues.xconfig 
#-------------------------------------------------------------
function enable_appstore()
{
    uclog "enalbe_appstore()"

    MacroName=$MACRO_ENABLE_APPSTORE
    sed -i bak "s/\/\/$MacroName/$MacroName/" $FeaturesPath
}

#-------------------------------------------------------------
#    copy crash report
#-------------------------------------------------------------
function enable_crash_log()
{
    uclog "enable_crash_log()"

    MacroName=$MACRO_ENABLE_CRASH_LOG
    sed -i bak "s/\/\/$MacroName/$MacroName/" $FeaturesPath
}

#-------------------------------------------------------------
#    copy enable_monkey_test 
#-------------------------------------------------------------
function enable_monkey_test()
{
   uclog "enable_monkey_test()"
   
    MacroName=$MACRO_ENABLE_MOKEY_TEST
    sed -i bak "s/\/\/$MacroName/$MacroName/" $FeaturesPath
}

enable_performance_test()
{
    uclog "enable performance test()"

    MacroName=$MACRO_ENABLE_PERFORMANCE_TEST
    sed -i bak "s/\/\/$MacroName/$MacroName/" $FeaturesPath
}

function enable_close_beta_test()
{
    uclog "enable_close_beta_test()"

    MacroName=$MACRO_ENABLE_CLOSE_BETA_TEST
    sed -i bak "s/\/\/$MacroName/$MacroName/" $FeaturesPath
}

function enable_iosurface()
{
   #open macro in Features.xconfig.
   uclog "enable_iosurface()"
   MacroName=$MACRO_ENABLE_IOSURFACE
   sed -i bak "s/\/\/$MacroName/$MacroName/" $FeaturesPath

   #add iosurface framework in pbxproj.

sed -i bak '/Begin PBXBuildFile section/a\
378DBA75166E2E0700B3F92A /* IOSurface.framework in Frameworks */ = {isa = PBXBuildFile; fileRef = 378DBA74166E2E0700B3F92A /* IOSurface.framework */; };
' $DIR_PROJECT/$projectfile/project.pbxproj

sed -i bak '/Begin PBXFileReference section/a\
378DBA74166E2E0700B3F92A /* IOSurface.framework */ = {isa = PBXFileReference; lastKnownFileType = wrapper.framework; name = IOSurface.framework; path = System/Library/PrivateFrameworks/IOSurface.framework; sourceTree = SDKROOT; };
' $DIR_PROJECT/$projectfile/project.pbxproj

build_phase_nr=`awk '/Begin PBXFrameworksBuildPhase section/ {print NR}' $DIR_PROJECT/$projectfile/project.pbxproj` 
build_phase_nr=`expr $build_phase_nr + 6`
sed -i bak "`echo $build_phase_nr`a\\
378DBA75166E2E0700B3F92A /* IOSurface.framework in Frameworks */,
" $DIR_PROJECT/$projectfile/project.pbxproj

frameworks_group_nr=`awk '/name = Frameworks;/ {print NR}' $DIR_PROJECT/$projectfile/project.pbxproj`
frameworks_group_nr=`expr $frameworks_group_nr - 3`
sed -i bak "`echo $frameworks_group_nr`a\\
378DBA74166E2E0700B3F92A /* IOSurface.framework */,
" $DIR_PROJECT/$projectfile/project.pbxproj

}

#-------------------------------------------------------------
#    copy init_MyNaviConfig 
#-------------------------------------------------------------
function copy_naviConfig()
{   
    uclog "copy_naviConfig()"

    SRC="init_MyNaviConfig_apps"
    DES="init_MyNaviConfig"

    for dir in `ls $DIR_PROJECT/../res_backup/Lang/ipad_intl`; do
        if [ $dir != "." ] && [ $dir != ".." ]; then
            cp -f $DIR_PROJECT/../res_backup/mynavi/ipad_int/$dir/$SRC  $DIR_PROJECT/../res_backup/Lang/ipad_intl/$dir/mynavi/$DES
        fi

    done
}

#-------------------------------------------------------------
#  adjust some variables after int_evn().
#-------------------------------------------------------------
function adjust_vars()
{
    ProductName=$DEFS_UC_PRO_NAME
    appName=$ProductName".app"
}

#-------------------------------------------------------------
#  copy and set config.ini for u-alliance.
#-------------------------------------------------------------
function set_ualliance_config()
{
    uclog "set_ualliance_config()"

    UConfig="config.ini"
    SrcConfigPath=$DIR_PROJECT/../$UConfig
    DesConfigPath=$DIR_PROJECT/../res/UCMobile/setting/$UConfig
    
    if [ ! -f $SrcConfigPath ]
    then
        uclog "config.ini not exist."
        exit 87
    fi

    cp $SrcConfigPath $DesConfigPath

    modify_set_value "ch" 123@some.com $DesConfigPath
    modify_set_value "btype" $DEFS_BTYPE $DesConfigPath
    modify_set_value "bmode" $DEFS_BMODE $DesConfigPath
    
}

#-------------------------------------------------------------
#  check the param vars. 
#  some params are exclusive.
#-------------------------------------------------------------
function check_params()
{
    uclog "check_params()"

#   DEF_TYP1, DEF_TYPE2, DEF_TYPE3 are exclusive.
    i=0

    if [ $DEF_TYPE1 = "true" ]; then
        i=$((i+1))
    fi

    if [ $DEF_TYPE2 = "true" ]; then    
        i=$((i+1))
    fi

    if [ $DEF_TYPE3 = "true" ]; then
        i=$((i+1))
    fi

    if [ ! $i = 1 ]; then
        uclog "i=$i"
        echo "WRONG Release/Debug/Distribution selecton!"
        exit 66
    fi

#   DEF_RUN1, DEF_RUN2, DEF_RUN3 are exclusive.
    j=0
    if [ $DEF_RUN1 = "true" ]; then
        j=$((j+1))
    fi
    
    if [ $DEF_RUN3 = "true" ]; then 
        j=$((j+1))
    fi  

    if [ $DEF_RUN4 = "true" ]; then 
        j=$((j+1))
    fi
    
    if [ ! $j = 1 ]; then 
        uclog "j=$j"
        echo "wrong iphoneos5.1/iphonesimulator5.1/iphonesimulator4.3 selection!"
        exit 67
    fi

}

#-------------------------------------------------------------
#  generate lp_help file
#-------------------------------------------------------------
gen_lp_help()
{
    chmod 755 $BaseDir/gen_lp_help.sh
    LpHelpSrcBaseDir="$BaseDir/BrowserShell_ios/ipad-cn/res_backup/lp_help_src/ipad_int"

    for dir in `ls $LpHelpSrcBaseDir`; do
        LpHelpSrcDir="$LpHelpSrcBaseDir/$dir"
        LpHelpTargetDir="$BaseDir/BrowserShell_ios/ipad-cn/res_backup/Lang/ipad_intl/$dir/help/"
        SoftInfoFile="$BaseDir/BrowserShell_ios/ipad-cn/SoftInfo.h"

        ./gen_lp_help.sh $LpHelpSrcDir $LpHelpTargetDir $SoftInfoFile
    done

}


#-------------------------------------------------------------
#  build the ucmobile/ucweb project
#-------------------------------------------------------------
build_ucweb()
{
    uclog ">>> $BuildScript : build_ucweb()"
   
#   DEF_TYP1, DEF_TYPE2, DEF_TYPE3 are exclusive.
#   DEF_RUN1, DEF_RUN2, DEF_RUN3 are exclusive.
#   So need to check that.
    check_params 

#   whether enable crash log.
    if [ $DEF_CRASH_LOG_ENABLE = "true" ]
    then
    enable_crash_log
    fi

#   whether enable mokey test.
    if [ $DEF_MONKEY_TEST_ENABLE = "true" ]
    then 
        enable_monkey_test
    fi

#   whether enable close beta test.
    if [ $DEF_CLOSE_BETA_TEST_ENABLE = "true" ]
    then
        enable_close_beta_test
    fi

    if [ $DEF_PERFORMANCE_TEST_ENABLE = "true" ]
    then
        enable_performance_test
    fi

#   iosurface
#   when DEF_IOSURFACE_ENABLE is open, and not built for simulator.
    if [ $DEF_IOSURFACE_ENABLE = "true" ] && [ $DEF_RUN3 != "true" ] && [ $DEF_RUN4 != "true" ]
    then
        enable_iosurface
    fi

    #generate sdkString.   
    if [ $DEF_RUN1 = "true" ]; then
        sdkString="iphoneos7.0" 
    fi

    if [ $DEF_RUN3 = "true" ]; then
        sdkString="iphonesimulator7.0" 
    fi

    if [ $DEF_RUN4 = "true" ]; then
        sdkString="iphonesimulator4.3" 
    fi
   
    #default sdk is 7.0.
    if [ -z $sdkString ]; then
        sdkString="iphoneos7.0"
    fi

    uclog "sdkString=$sdkString"

    #generate typeString.    
    if [ $DEF_TYPE1 = "true" ]; then
        typeString="Release" 
    fi

    if [ $DEF_TYPE2 = "true" ]; then
        typeString="Debug" 
    fi

    if [ $DEF_TYPE3 = "true" ]; then
        typeString="Distribution" 
    fi

    #default type:Release.
    if [ -z $typeString ]; then
        typeString="Release"
    fi

    export BUILD_TYPE_STRING=$typeString
    uclog $BUILD_TYPE_STRING    

    ####---- real clean & build action
    (cd $DIR_PROJECT
     #xcodebuild clean -project $projectfile -sdk $sdkString -configuration $typeString

    codeSignEntitlementsString=""
    if [ $DEF_ENTITLEMENTS = "true" ]; then
        codeSignEntitlementsString="UCBrowserHD_int.entitlements"
    fi

#    distribution: need to remove MobieInstallation.framework, enable ENABLE_APPSTORE, copy init_MyNaviConfig.
     if [ $DEF_TYPE3 = "true" ]; then
         remove_framework
         codeSignEntitlementsString=""
         enable_appstore
         copy_naviConfig
     fi

#    copy file for u alliance.
     if [ "$DEF_U_ALLIANCE" = "true" ]; then 

#       u_alliance has the same config with jail broken release.
#       so it's different with distribuion.
        if [ $DEF_TYPE3 = "true" ];then 
             uclog "distribution can't be build into u_alliance package."
             exit 21
        fi

#       set config.ini for u-alliance.
        set_ualliance_config
     fi

#    judge architecture
     archString="i386"
     if [ ${sdkString:6:2} = "os" ]; then
          archString="armv7"
     fi
     uclog $archString

     modify_bundle "$DIR_PROJECT/../$infolistfile"

     ~/bin/selectXcode5
     xcodebuild -project $projectfile -sdk $sdkString -arch $archString -configuration $typeString CODE_SIGN_ENTITLEMENTS="${codeSignEntitlementsString}" > build.log
    )

    # an error occured, zip logs instead
    if [ $? != 0 ]; then
          uclog "an build error occured,"
          mkdir $DIR_OUT
          mv  -f  $DIR_PROJECT/build.log  $DIR_OUT
          cp $DIR_SRC_ROOT/build.ini $DIR_OUT
          exit 139
    fi
}

#-------------------------------------------------------------
#  zip UCBrowserHD.app into ~.ipa, ~.zip
#-------------------------------------------------------------
zip_process()
{
    uclog "BuildScript : zip_process()"
    uclog "start to zip .app into .ipa/.zip ..."
    
#   for os.
    if [ $DEF_RUN1 = "true" ]
    then
        if [ $DEF_TYPE1 = "true" ]
        then
             appResultDir=$DIR_BUILD_RELEASE
             LibraryResultDir=$DIR_UCWEB_BUILD_RELEASE
             LibraryResultDir_Shell=$DIR_BUILD_RELEASE
        elif [ $DEF_TYPE2 = "true" ]
        then 
             appResultDir=$DIR_BUILD_DEBUG
             LibraryResultDir=$DIR_UCWEB_BUILD_DEBUG
             LibraryResultDir_Shell=$DIR_BUILD_DEBUG
        elif [ $DEF_TYPE3 = "true" ]
        then
             appResultDir=$DIR_BUILD_DISTRIBUTION
             LibraryResultDir=$DIR_UCWEB_BUILD_DISTRIBUTION
             LibraryResultDir_Shell=$DIR_BUILD_DISTRIBUTION
        fi
#   for simulator.       
    elif [ $DEF_RUN3 = "true" -o $DEF_RUN4 = "true" ]
    then
        if [ $DEF_TYPE1 = "true" ]
        then
            appResultDir=$DIR_BUILD_RELEASE_SIM
            LibraryResultDir=$DIR_UCWEB_BUILD_RELEASE_SIM
            LibraryResultDir_Shell=$DIR_BUILD_RELEASE_SIM
        elif [ $DEF_TYPE2 = "true" ]
        then 
            appResultDir=$DIR_BUILD_DEBUG_SIM
            LibraryResultDir=$DIR_UCWEB_BUILD_DEBUG_SIM
            LibraryResultDir_Shell=$DIR_BUILD_DEBUG_SIM
        elif [ $DEF_TYPE3 = "true" ]
        then
            appResultDir=$DIR_BUILD_DISTRIBUTION_SIM
            LibraryResultDir=$DIR_UCWEB_BUILD_DISTRIBUTION_SIM
            LibraryResultDir_Shell=$DIR_BUILD_DISTRIBUTION_SIM
        fi
    fi
       
        uclog "appResultDir:$appResultDir"
        appResult=$appResultDir/$appName

        if [ -d $DIR_RESULT ]
        then
            rm -rf $DIR_RESULT
        fi

        mkdir $DIR_RESULT

        uclog "copy .app .dSYM .a into $DIR_RESULT ..."

        mv $DIR_PROJECT/../"iTunesArtwork" $DIR_RESULT
        mv  $appResultDir/*.app  $DIR_RESULT
        mv  $appResultDir/*.dSYM $DIR_RESULT
        mv    $LibraryResultDir/*.a    $DIR_RESULT
        mv  $LibraryResultDir_Shell/*.a $DIR_RESULT

        ####---- start packing
        if [ ! -x ./ucweb.sh ]; then
            chmod +x ./ucweb.sh
        fi

        uclog "exectue ucweb.sh now."
        ./ucweb.sh 
 }

#-------------------------------------------------------------
#  copy all the zip file into out directory.
#-------------------------------------------------------------
output_process()
{
    uclog ">>> $BuildScript : output_process()"
    uclog "output to OUT dir"    

    rm -rf $DIR_OUT
    mkdir $DIR_OUT

#   copy the package.
    mv $DIR_RESULT/*.zip $DIR_OUT  
#   copy the build config file.
    cp $DIR_SRC_ROOT/build.ini $DIR_OUT
    
    mv $DIR_RESULT/*.ipa $DIR_OUT
    
    if [ $DEF_SYMBOL_FILE = "true" ]; then
        mv $DIR_RESULT/*.dSYM $DIR_OUT
    fi
    
    if [ $DEF_PACKAGE_TYPE = "true" ]; then
        mv $DIR_RESULT/*.a $DIR_OUT
    fi

#    rm -rf $DIR_RESULT
}


#-------------------------------------------------------
#-------------------------------------------------------
#-------------------------------------------------------
#               the main script flow is here
#-------------------------------------------------------
#------------------------------------------------------- 

uclog "Start building UCBrowserHD ..."

init_vars

init_env

adjust_vars

alter_softInfo

alter_proInfo

enable_layout_test

enable_remote_inspector

gen_lp_help

build_ucweb

zip_process

output_process

uclog "Auto package process is finished now!"

