#!/bin/bash
if [ $# -ne 1 ];then
    echo "usage: $0 conf_file"
    exit
fi
conf_file=$1
if [ ! -f $conf_file ];then
    echo "$conf_file is not a file"
    exit
fi

conf_name="${conf_file##*/}"
conf_dir="${conf_file%/*}"

#生成enum文件

enumfile="${conf_dir}/${conf_name}_FSMEnum.h"

echo "//" > $enumfile
echo "// "${conf_name}_FSMEnum.h >>  $enumfile
echo "//" >> $enumfile
echo "// This is created by script" >>$enumfile
echo "// It can't be modified" >>$enumfile
echo "//" >>$enumfile
echo "" >> $enumfile
echo "" >> $enumfile
echo "#ifndef "$conf_name"_FSMEnum_h" >> $enumfile
echo "#define "$conf_name"_FSMEnum_h" >> $enumfile
echo "" >> $enumfile
echo "" >> $enumfile
echo "#import \"DefFSMEnum.h\"" >> $enumfile
echo "" >> $enumfile
echo "" >> $enumfile

awk '{
if (NF == 4 && !(substr($1,0,1)=="#"))
    {
        s_dic[$1] ++;
        s_dic[$2] ++;
        e_dic[$3] ++;
    }
}
END{
print "typedef NS_OPTIONS(NSUInteger, '$conf_name'_State){";
first = 1;
for (key in s_dic)
    {
        if (key == "IDLE" || key == "OTHERSTATE_BEGIN")
            continue;
        if (first == 1)
            {
                printf("\t%s = OTHERSTATE_BEGIN", key);
                first = 2;
            }
        else
            {
                printf(",\n\t%s", key);
            }
    }
print "\n};";

print "";

first = 1;
print "typedef NS_OPTIONS(NSUInteger, '$conf_name'_InternalEvent){";
for (key in e_dic)
    {
        if (key == "TIME_OUT" || key == "INTERNAL_EVENT_BEGIN")
            continue;
        if (first == 1)
            {
               if(!((substr(key,0,1)=="@") || (substr(key,0,1)=="$")))
                    {
                        printf("\t%s = INTERNAL_EVENT_BEGIN", key);
                        first = 2;
                    }   
            }
        else
            {
                if(!((substr(key,0,1)=="@") || (substr(key,0,1)=="$")))
                    {
                        printf(",\n\t%s", key);
                    }
            }
        }
print "\n};";
}
' $conf_file >> $enumfile

echo "" >> $enumfile
echo "#endif" >> $enumfile


#生成FSMHander.h
fsmhanderfile="${conf_dir}/${conf_name}_FSMHander.h"
echo "//" > $fsmhanderfile
echo "// "${conf_name}_FSMHander.h >> $fsmhanderfile
echo "//" >> $fsmhanderfile
echo "// This is created by script" >> $fsmhanderfile
echo "// It can't be modified" >> $fsmhanderfile
echo "//" >> $fsmhanderfile
echo "" >> $fsmhanderfile
echo "" >> $fsmhanderfile
echo "#import <Foundation/Foundation.h>" >> $fsmhanderfile
echo "#import \"UCFSMEvent.h\"" >> $fsmhanderfile
echo "#import \"FSMAction.h\"" >> $fsmhanderfile 
echo "" >> $fsmhanderfile
awk '{
if (NF == 4 && !(substr($1,0,1)=="#"))
    {
        h_dic[$4] ++;
    }
}
END{
    for(key in h_dic)
    {
        printf("void %s_'$conf_name'(id<FSMAction>fsm, id<UCFSMEvent> event);\n\n", key);
    }
}
' $conf_file >> $fsmhanderfile

#生成FSMhander.mm如果不存在的话
fsmhanderfile_mm="${conf_dir}/${conf_name}_FSMHander.mm"
if [ ! -f $fsmhanderfile_mm ] ;then

	echo "//" > $fsmhanderfile_mm
	echo "// "${conf_name}_FSMHander.mm >> $fsmhanderfile_mm
	echo "//" >> $fsmhanderfile_mm
	echo "// This is created by script" >> $fsmhanderfile_mm
	echo "// It can be modified" >> $fsmhanderfile_mm
	echo "//" >> $fsmhanderfile_mm
	echo "" >> $fsmhanderfile_mm
	echo "" >> $fsmhanderfile_mm
	echo "#import \"${conf_name}_FSMHander.h\"" >> $fsmhanderfile_mm
	echo "" >> $fsmhanderfile_mm
	awk '{
    if (NF == 4 && !(substr($1,0,1)=="#"))
	    {
	        h_dic[$4] ++;
	    }
	}
	END{
	    for(key in h_dic)
	    {
	        printf("void %s_'$conf_name'(id<FSMAction>fsm, id<UCFSMEvent> event)\n{\n\n}\n\n", key);
	    }
	}
	' $conf_file >> $fsmhanderfile_mm
	
fi


#生成ConcreteFSM文件
#concreteFSM.h
concreteFSM_h="${conf_dir}/${conf_name}FSM.h"

echo "//" > $concreteFSM_h
echo "// "${conf_name}FSM.h >> $concreteFSM_h
echo "//" >> $concreteFSM_h
echo "// This is created by script" >> $concreteFSM_h
echo "// It can't be modified" >> $concreteFSM_h
echo "//" >> $concreteFSM_h
echo "" >> $concreteFSM_h
echo "" >> $concreteFSM_h

echo "#import \"FSM.h\"" >> $concreteFSM_h
echo "" >> $concreteFSM_h
echo "@interface ${conf_name}FSM : FSM" >> $concreteFSM_h
echo "" >> $concreteFSM_h
echo "- (id)init${conf_name}FSMWithEntityController:(id)controller;" >> $concreteFSM_h
echo "" >> $concreteFSM_h

echo "@end" >> $concreteFSM_h

#concreteFSM.mm
concreteFSM_mm="${conf_dir}/${conf_name}FSM.mm"

echo "//" > $concreteFSM_mm
echo "// "${conf_name}FSM.mm >> $concreteFSM_mm
echo "//" >> $concreteFSM_mm
echo "// This is created by script" >> $concreteFSM_mm
echo "// It can't be modified" >> $concreteFSM_mm
echo "//" >> $concreteFSM_mm
echo "" >> $concreteFSM_mm
echo "" >> $concreteFSM_mm

echo "#import \"${conf_name}FSM.h\"" >> $concreteFSM_mm
echo "#import \"State.h\"" >> $concreteFSM_mm
echo "#import \"InternalEvent.h\"" >> $concreteFSM_mm
echo "#import \"Transition.h\"" >> $concreteFSM_mm
echo "#import \"${conf_name}_FSMHander.h\"" >> $concreteFSM_mm
echo "#import \"${conf_name}_FSMEnum.h\"" >> $concreteFSM_mm

echo "" >> $concreteFSM_mm
echo "@implementation ${conf_name}FSM" >> $concreteFSM_mm
echo "" >> $concreteFSM_mm


echo "- (NSArray *)getTransitionTable_${conf_name}" >> $concreteFSM_mm
echo "{" >> $concreteFSM_mm
echo "    NSMutableArray *retArray = [[NSMutableArray alloc] initWithCapacity:10];" >> $concreteFSM_mm
echo "" >> $concreteFSM_mm
awk '
{
if (NF == 4 && !(substr($1,0,1)=="#"))
{
    print "    //"$1, $2, $3, $4;
    print "    {";
    print "        State *sState = [State stateWithID:"$1"];";
    print "        State *tState = [State stateWithID:"$2"];";
    if(substr($3,0,1)=="@")
    {
        print "        NSString *event = @\""substr($3,2)"\";";
    }
    else if (substr($3,0,1)=="$")
    {
        print "        NSNotification *event = [NSNotification notificationWithName:@\""substr($3,2)"\" object:nil];";
    }
    else
    {
        print "        InternalEvent *event = [InternalEvent internalEventWithId:"$3"];";
    }
    print "        Transition* transition = [Transition transitionWithSourceState:sState targetState:tState event:event handler:"$4"_'$conf_name'];";
    print "        [retArray addObject:transition];";
    print "    }\n";
}
}
' $conf_file >> $concreteFSM_mm

echo "    return [retArray autorelease];" >> $concreteFSM_mm
echo "}" >> $concreteFSM_mm

echo "" >> $concreteFSM_mm
echo "" >> $concreteFSM_mm


echo "- (id)init${conf_name}FSMWithEntityController:(id)controller" >> $concreteFSM_mm
echo "{" >> $concreteFSM_mm
echo "    self = [super init];" >> $concreteFSM_mm
echo "    if (self)" >> $concreteFSM_mm
echo "    {" >> $concreteFSM_mm
echo "        [self initFSMWithTransitionTable:[self getTransitionTable_${conf_name}] entityController:controller];" >> $concreteFSM_mm
echo "    }" >> $concreteFSM_mm
echo "" >> $concreteFSM_mm
echo "    return self;" >> $concreteFSM_mm
echo "}" >> $concreteFSM_mm
echo "" >> $concreteFSM_mm
echo "@end" >> $concreteFSM_mm


