/*
 * EvernoteSession.m
 * evernote-sdk-ios
 *
 * Copyright 2012 Evernote Corporation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

function ContentVeil() {
  "use strict";

  // @TODO: save() and restore() aren't properly used here, so if we do things like add transforms in founctions,
  // we probably break other functions' notion of how to render things.

  // Wanglei:add
  var bottomToBodyTopDistance=0;
  var outlineEle=null;
  //
  var veil = document.createElement("div");
  var inner = document.createElement("div");
  veil.appendChild(inner);

  veil.style.boxSizing = "border-box";
  veil.style.borderStyle = "solid";
  veil.style.borderColor = "rgba(0, 0, 0, 0.7)";

  inner.style.border = "4px solid rgba(255, 255, 0, 0.7)";
  inner.style.height = "100%";
  inner.style.boxSizing = "border-box";

  // We keep a record of what we're currently showing (at least in some cases) so that we can update it in case the
  // state of the page changes (like if the user scrolls).
  var currentlyShownRect = null;
  var currentRectOffsetTop = 0;
  var currentRectOffsetLeft = 0;
  var currentlyStatic = false;

  function reset() {
    currentlyShownRect = null;
    currentRectOffsetTop = 0;
    currentRectOffsetLeft = 0;

    showElements("embed");
    showElements("iframe");
    
    veil.style.position = "absolute"; // Wanglei:change fixed to absolue for scroll bug 
    veil.style.top = "0px";
    veil.style.left = "0px";
    veil.style.zIndex = "9999999999999990";

    blank();
  }

  function blank() {
    veil.style.height = document.documentElement.clientHeight + "px";
    veil.style.width = document.documentElement.clientWidth + "px";
  }

  function gray() {
    show();
    inner.style.display = "none";
    veil.style.backgroundColor = veil.style.borderColor;
  }

  function show() {
    inner.style.display = "";
    veil.style.backgroundColor = "";
    if (!veil.parentNode) {
      document.documentElement.appendChild(veil);
    }
  }

  function hide() {
    if (veil.parentNode) {
      veil.parentNode.removeChild(veil);
    }
  }

  // Makes a rectangle bigger in all directions by the number of pixels specified (or smaller, if 'amount' is 
  // negative). Returns the new rectangle.
  function expandRect(rect, amount) {
    return {
      top: (rect.top - amount),
      left: (rect.left - amount),
      bottom: (rect.bottom + amount),
      right: (rect.right + amount),
      width: (rect.width + (2 * amount)),
      height: (rect.height + (2 * amount))
    };
  }

  // DrawStroke is obsolete, it is now always "true".
  function revealRect(rect, drawStroke, staticView) {

    // Save this info.
    currentlyShownRect = rect;
    currentRectOffsetTop = document.body.scrollTop;
    currentRectOffsetLeft = document.body.scrollLeft;
    currentlyStatic = staticView;

    // We expand the rectangle for two reasons. 
    // 1) we want to expand it by the width of the stroke, so that when we draw out outline, it doesn't overlap our
    // content.
    // 2) We want to leave a little extra room around the content for aesthetic reasons.
    rect = expandRect(rect, 8);
    var x = rect.left;
    var y = rect.top;
    var width = rect.width;
    var height = rect.height;

    var veilWidth = veil.style.width.replace("px", "");
    var veilHeight = veil.style.height.replace("px", "");

    var offScreen = false;
    if (y + height < 0) {
      offScreen = true;
    }
    else if (y > veilHeight) {
      offScreen = true;
    }
    else if (x + width < 0) {
      offScreen = true;
    }
    else if (x > veilWidth) {
      offScreen = true;
    }

    if (offScreen) {
      veil.style.borderLeftWidth = veilWidth + "px";
      veil.style.borderTopWidth = veilHeight + "px";
      veil.style.borderRightWidth = "0px";
      veil.style.borderBottomWidth = "0px";
      inner.style.display = "none";
      return;
    }

    inner.style.display = "block";
    veil.style.borderLeftWidth = Math.max(x, 0) + "px";
    veil.style.borderTopWidth = Math.max(y, 0) + "px";
    veil.style.borderRightWidth = Math.max((veilWidth - x - width), 0) + "px";
    // Wanglei:add
    var distance=0;
    if ( bottomToBodyTopDistance.top <0){
       distance= bottomToBodyTopDistance.bottom- bottomToBodyTopDistance.top;
    }
    else{
       distance= bottomToBodyTopDistance.bottom;
    }
    veil.style.borderBottomWidth =(document.body.clientHeight-distance)+ 'px';// Math.max((veilHeight - y - height), 0) + "px";
    veil.style.height =document.body.clientHeight+'px';
    // Wanglei:end
  }

  function revealStaticRect(rect, drawStroke) {
    revealRect(rect, drawStroke, true);
  }
  function outlineElement(element, scrollTo) {
    // See notes in Preview.js for why we use this method instead of just calling element.getBoundingClientRect().
    // Wanglei:add
    outlineEle=element;
    // Wanglei:end
    var rect = contentPreview.computeDescendantBoundingBox(element);
    if (rect) {

      var mutableRect = {
        top: rect.top,
        bottom: rect.bottom,
        left: rect.left,
        right: rect.right,
        width: rect.width,
        height: rect.height,
      }
      // Wanglei:add
      bottomToBodyTopDistance=mutableRect; 
      // Wanglei:end
      // We don't want to adjust ourselves into odd positions if the page is scrolled.
      var sLeft = document.body.scrollLeft;
      var sTop = document.body.scrollTop;

      var BORDER_MIN = 9;
      if (mutableRect.left < (BORDER_MIN - sLeft)) {
        mutableRect.width -= (BORDER_MIN - sLeft) - mutableRect.left;
        mutableRect.left = (BORDER_MIN - sLeft);
      }
      if (mutableRect.top < (BORDER_MIN - sTop)) {
        mutableRect.height -= (BORDER_MIN - sTop) - mutableRect.top;
        mutableRect.top = (BORDER_MIN - sTop);
      }

      // Get the wider of our two possible widths.
      var width = Math.max(document.body.scrollWidth, window.innerWidth);

      if (mutableRect.right > (width - BORDER_MIN - sLeft)) {
        mutableRect.right = (width - BORDER_MIN - sLeft);
        mutableRect.width = mutableRect.right - mutableRect.left;
      }

        
      reset();
        
      revealRect(mutableRect, true);
        
      if (scrollTo) {
        element.scrollIntoViewIfNeeded(true);
        // Use the following if this makes it into Firefox or other Gecko-based browsers:
        // element.scrollIntoView(false);
      }
      hideElements("embed", element);
      hideElements("iframe", element);
      show();
        
      // Wanglei
      inner.style.height = (mutableRect.height)+'px';
    }
    else {
      console.warn("Couldn't create rectangle from element: " + element.toString());
    }
  }

  function hideElements (tagName, exceptInElement) {
    var els = document.getElementsByTagName(tagName);
    for (var i = 0; i < els.length; i++) {
      els[i].enSavedVisibility = els[i].style.visibility;
      els[i].style.visibility = "hidden";
    }
    showElements(tagName, exceptInElement);
  }

  function showElements (tagName, inElement) {
    if (!inElement) {
      inElement = document;
    }
    var els = inElement.getElementsByTagName(tagName);
    for (var i = 0; i < els.length; i++) {
      if (typeof els[i].enSavedVisibility !== "undefined") {
        els[i].style.visibility = els[i].enSavedVisibility;
        delete els[i].enSavedVisibility;
      }
    }
  }

  // If we're currently showing a rectangle, and it's not static, we'll redraw on scroll.
//  window.addEventListener("scroll", function(e) { Wanglei:changed
    if (currentlyShownRect && !currentlyStatic) {
      var rect = {
        top: currentlyShownRect.top,
        bottom: currentlyShownRect.bottom,
        left: currentlyShownRect.left,
        right: currentlyShownRect.right,
        width: currentlyShownRect.width,
        height: currentlyShownRect.height
      };

      var vert = document.body.scrollTop - currentRectOffsetTop;
      var horiz = document.body.scrollLeft - currentRectOffsetLeft;

      if (!vert && !horiz) {
        return;
      }

      rect.top -= vert;
      rect.bottom -= vert;
      rect.left -= horiz;
      rect.right -= horiz;

      blank();
      revealRect(rect);
    }
 // });  Wanglei:end
  // Wanglei:add
  var flag=0;
  var sto=0;
  function setVeilHeight(){
        //alert('bottomToBodyTopDistance.top:'+bottomToBodyTopDistance.top+'     bottomToBodyTopDistance.bottomd:'+bottomToBodyTopDistance.bottom)
    if (document.body.clientHeight>parseInt( veil.style.height)){
        var distance=0;
        //当页面不在最顶端位置的时候,此时bottom值需要和top的值一起来得到最终的值
        if ( bottomToBodyTopDistance.top <0){
           distance= bottomToBodyTopDistance.bottom- bottomToBodyTopDistance.top;
        }
        else{
           distance= bottomToBodyTopDistance.bottom;
        }
        veil.style.borderBottomWidth =(document.body.clientHeight-distance)+ 'px';// 加上8px否则黄色框和黑色之间留有空隙
        veil.style.height =document.body.clientHeight+'px';// Math.max((veilHeight - y - height), 0) + "px";
        clearTimeout(sto);
        return ;
        }
    if(++flag<6){
       sto=setTimeout(setVeilHeight,500);
       }
    else{
      clearTimeout(sto);
    }
  }
  setVeilHeight();
  
  window.addEventListener("orientationchange", function(e) {
      if(outlineEle!==null){
        var rect = contentPreview.computeDescendantBoundingBox(element);
        bottomToBodyTopDistance=rect;
        setVeilHeight();
      }
  });
  // Wanglei:end
  // Public API:
  this.reset = reset;
  this.show = show;
  this.gray = gray;
  this.hide = hide;
  this.revealRect = revealRect;
  this.revealStaticRect = revealStaticRect;
  this.outlineElement = outlineElement;
  this.expandRect = expandRect;
}

